from typing import List, Optional

from opennem.api.stats.schema import ScadaDateRange
from opennem.core.normalizers import normalize_duid
from opennem.schema.network import NetworkNEM, NetworkSchema
from opennem.schema.time import TimeInterval, TimePeriod


def station_id_case(station_codes: List[str]) -> str:
    return ",".join(
        ["'{}'".format(i) for i in map(normalize_duid, station_codes)]
    )


def observation_query(
    station_codes: List[str],
    interval: TimeInterval,
    network: NetworkSchema = NetworkNEM,
    timezone: str = "UTC",
    period: Optional[TimePeriod] = None,
    scada_range: Optional[ScadaDateRange] = None,
    year: Optional[str] = None,
):

    timezone = network.get_timezone(postgres_format=True)

    if not timezone:
        timezone = "UTC"

    __query = """
    SET SESSION TIME ZONE '{timezone}';

    select
        time_bucket_gapfill('{trunc}', observation_time) as observation_time,
        fs.station_id,
        interpolate(avg(fs.temp_air)) as temp_avg,
        interpolate(min(fs.temp_air)) as temp_min,
        interpolate(max(fs.temp_air)) as temp_max
    from bom_observation fs
    where
        fs.station_id in ({station_codes})
        and fs.observation_time <= {date_end}
        and fs.observation_time >= {date_start_condition}
    group by 1, 2
    order by 1 desc, 2 desc
    """

    date_end = "now()"

    if scada_range:
        date_end = scada_range.get_end_sql()

    if period:
        date_start_condition = "{date_end}::timestamp - '{period}'::interval".format(
            date_end=date_end, period=period.period_sql
        )

    if year:
        date_start_condition = "'{year}-01-01'::date".format(year=year)

    if not period and not year:
        raise Exception("Require period or year")

    query = __query.format(
        station_codes=station_id_case(station_codes),
        trunc=interval.interval_sql,
        timezone=timezone,
        date_start_condition=date_start_condition,
        date_end=date_end,
    )

    return query



from datetime import datetime, timedelta
from textwrap import dedent
from typing import Optional

from opennem.api.stats.controllers import get_scada_range
from opennem.api.stats.schema import ScadaDateRange
from opennem.api.time import human_to_interval, human_to_period
from opennem.schema.network import NetworkSchema, NetworkWEM
from opennem.schema.time import TimeInterval, TimePeriod


def wem_demand_all(network_code: str = "WEM"):

    __query = """
        SET SESSION TIME ZONE '+08';

        select(
            date_trunc('month', t.trading_interval),
            t.price,
            t.generated
        ) from (
            select
                time_bucket_gapfill('1 day', bs.trading_interval,
                round(wbs.price, 2) as price,
                round(wbs.generation_total, 2) as generated
            from balancing_summary bs
            where bs.network_id='{network_code}'
        ) as t
        group by 1,
        order by 1 desc
    """

    query = __query.format(network_code=network_code)

    return query


def power_network_fueltech_query(
    interval: TimeInterval,
    network: NetworkSchema = NetworkWEM,
    period: TimePeriod = human_to_period("7d"),
    network_region: Optional[str] = None,
) -> str:

    scada_range: ScadaDateRange = get_scada_range(network=network)

    timezone = network.timezone_database

    __query = """
        select
            t.trading_interval at time zone '{timezone}',
            t.fueltech_code,
            sum(t.facility_power)
        from (
            select
                time_bucket_gapfill('{trunc}', trading_interval) AS trading_interval,
                coalesce(
                    avg(fs.generated), 0
                ) as facility_power,
                fs.facility_code,
                ft.code as fueltech_code
            from facility_scada fs
            join facility f on fs.facility_code = f.code
            join fueltech ft on f.fueltech_id = ft.code
            where
                fs.network_id='{network_code}'
                and f.fueltech_id is not null
                and fs.trading_interval <= '{date_max}'
                and fs.trading_interval > '{date_min}'
                {network_region_query}
            group by 1, 3, 4
        ) as t
        group by 1, 2
        order by 1 desc
    """

    network_region_query = ""

    if network_region:
        network_region_query = f"and f.network_region='{network_region}'"

    date_max = scada_range.get_end()
    date_min = scada_range.get_end() - timedelta(days=7)

    query = dedent(
        __query.format(
            network_code=network.code,
            trunc=interval.interval_sql,
            period=period.period_sql,
            network_region_query=network_region_query,
            timezone=timezone,
            date_max=date_max,
            date_min=date_min,
        )
    )

    return query


def energy_network_fueltech_daily_query(
    network: NetworkSchema,
    year: int,
    interval: TimeInterval,
    network_region: Optional[str] = None,
) -> str:
    """
        Get Energy for a network or network + region
        based on a year
    """

    __query = """
        select
            date_trunc('day', t.trading_interval at time zone '{timezone}') as trading_day,
            t.fueltech_id,
            energy_sum(t.generated, '1 day') * interval_size('1 day', count(t.generated)) / 1000 as facility_energy,
            energy_sum(t.generated, '1 day') * interval_size('1 day', count(t.generated)) * avg(t.price) as price
        from
            (select
                time_bucket_gapfill('{trunc}', fs.trading_interval) as trading_interval,
                f.fueltech_id,
                count(fs.generated) as sample_count,
                avg(fs.generated) as generated,
                avg(bs.price) as price
            from facility_scada fs
                left join facility f on fs.facility_code = f.code
                left join balancing_summary bs on
                    date_trunc('hour', bs.trading_interval) = date_trunc('hour', fs.trading_interval)
                    and bs.network_id='{network_code}'
            where
                fs.network_id='{network_code}'
                and f.fueltech_id is not null
                and fs.trading_interval <= '{year_max}'
                and fs.trading_interval >= '{year_min}'
                {network_region_query}
            group by
                1,
                f.code,
                f.fueltech_id
            ) as t
        group by 1, 2
        order by
            trading_day desc;
    """

    timezone = network.timezone_database
    offset = network.get_timezone(postgres_format=True)
    scada_range: ScadaDateRange = get_scada_range(network=network)

    if not timezone:
        timezone = "UTC"

    # might have to do +08 times
    year_max = "{}-12-31 23:59:59{}".format(year, offset)
    year_min = "{}-01-01 00:00:00{}".format(year, offset)

    if year == datetime.now().year:
        year_max = scada_range.get_end()

    network_region_query = ""

    if network_region:
        network_region_query = f"and f.network_region='{network_region}'"

    query = dedent(
        __query.format(
            network_code=network.code,
            trunc="{} minutes".format(network.interval_size),
            year_min=year_min,
            year_max=year_max,
            network_region_query=network_region_query,
            timezone=timezone,
        )
    )

    return query


def market_value_daily_query(
    year: int,
    network: NetworkSchema = NetworkWEM,
    network_region: Optional[str] = None,
) -> str:

    scada_range: ScadaDateRange = get_scada_range(network=network)

    __query = """
        SET SESSION TIME ZONE '+08';

        select
            time_bucket_gapfill('1 day', fs.trading_interval) AS trading_day,
            energy_sum(generated, '1 day') * interval_size('1 day', count(generated))  * avg(bs.price) as energy_interval,
            f.fueltech_id
        from facility_scada fs
        left join facility f on fs.facility_code = f.code
        join balancing_summary bs on fs.trading_interval::date = bs.trading_interval::date
        where
            f.fueltech_id is not null and
            fs.network_id='WEM' and
            fs.trading_interval <= {date_max} and
            fs.trading_interval >= {date_min}
        group by 1, f.fueltech_id
        order by 1 desc, 2 asc
    """

    network_region_query = ""

    if network_region:
        network_region_query = f"and f.network_region='{network_region}'"

    date_min = f"'{year}-01-01'"
    date_max = f"'{year}-12-31'"

    if year == datetime.now().year:
        date_max = scada_range.get_end_sql()

    if year == scada_range.start.year:
        date_min = scada_range.get_start_sql()

    query = __query.format(
        network_code=network.code,
        date_min=date_min,
        date_max=date_max,
        network_region_query=network_region_query,
    )

    return query


def wem_market_value_all_query(network_code: str = "WEM",) -> str:

    scada_range: ScadaDateRange = get_scada_range(network=NetworkWEM)

    __query = """
        SET SESSION TIME ZONE '+08';

        select
            date_trunc('month', t.trading_day),
            sum(t.energy_interval),
            t.fueltech_id
        from (
            select
                time_bucket_gapfill('1 day', fs.trading_interval) AS trading_day,
                energy_sum(generated, '1 day') * interval_size('1 day', count(generated)) * avg(bs.price) as energy_interval,
                f.fueltech_id
            from facility_scada fs
            left join facility f on fs.facility_code = f.code
            join balancing_summary bs on fs.trading_interval::date = bs.trading_interval::date
            where
                f.fueltech_id is not null and
                fs.network_id='{network_code}' and
                extract('year' from fs.trading_interval) = '2020' and
                fs.trading_interval <= {date_max} and
                fs.trading_interval >= {date_min}
            group by 1, 3
        ) as t
        group by 1, 3
        order by 1 desc, 3 asc;
    """

    query = __query.format(
        network_code=network_code,
        date_min=scada_range.get_start_sql(),
        date_max=scada_range.get_end_sql(),
    )

    return query



from typing import Optional

from opennem.api.export.queries import (
    energy_network_fueltech_daily_query,
    market_value_daily_query,
    power_network_fueltech_query,
    wem_market_value_all_query,
)
from opennem.api.stats.controllers import get_scada_range, stats_factory
from opennem.api.stats.schema import DataQueryResult, OpennemDataSet
from opennem.api.time import human_to_interval, human_to_period
from opennem.api.weather.queries import observation_query
from opennem.core.networks import network_from_network_code
from opennem.core.units import get_unit
from opennem.db import get_database_engine
from opennem.schema.time import TimePeriod


def weather_daily(year: int, network_code: str, station_code: str):
    station_codes = []

    if station_code:
        station_codes = [station_code]

    engine = get_database_engine()
    interval = human_to_interval("1d")
    network = network_from_network_code(network_code)
    units = get_unit("temperature_mean")
    scada_range = get_scada_range(network=network)

    query = observation_query(
        station_codes=station_codes,
        interval=interval,
        network=network,
        scada_range=scada_range,
        year=year,
    )

    with engine.connect() as c:
        row = list(c.execute(query))

    temp_avg = [
        DataQueryResult(
            interval=i[0], group_by=i[1], result=i[2] if len(i) > 1 else None
        )
        for i in row
    ]

    temp_min = [
        DataQueryResult(
            interval=i[0], group_by=i[1], result=i[3] if len(i) > 1 else None
        )
        for i in row
    ]

    temp_max = [
        DataQueryResult(
            interval=i[0], group_by=i[1], result=i[4] if len(i) > 1 else None
        )
        for i in row
    ]

    if len(temp_avg) < 1:
        raise Exception("No results from query: {}".format(query))

    stats = stats_factory(
        stats=temp_avg,
        units=units,
        network=network,
        interval=interval,
        code="bom",
        group_field="temperature",
    )

    stats.data += stats_factory(
        stats=temp_min,
        units=get_unit("temprature_min"),
        network=network,
        interval=interval,
        code="bom",
        group_field="temperature",
    ).data

    stats.data += stats_factory(
        stats=temp_max,
        units=get_unit("temprature_max"),
        network=network,
        interval=interval,
        code="bom",
        group_field="temperature",
    ).data

    return stats


def power_week(network_code: str = "WEM"):
    engine = get_database_engine()

    network = network_from_network_code(network_code)
    interval = human_to_interval("{}m".format(network.interval_size))
    period = human_to_period("7d")
    units = get_unit("power")
    network_region = None

    query = power_network_fueltech_query(
        network=network, period=period, interval=interval
    )

    # print(query)

    with engine.connect() as c:
        row = list(c.execute(query))

    stats = [
        DataQueryResult(
            interval=i[0], result=i[2], group_by=i[1] if len(i) > 1 else None
        )
        for i in row
    ]

    if len(stats) < 1:
        raise Exception("No results from query: {}".format(query))

    result = stats_factory(
        stats,
        code=network.code,
        network=network,
        interval=interval,
        period=period,
        units=units,
        region=network_region,
        fueltech_group=True,
    )

    return result


def energy_fueltech_daily(
    year: int,
    network_code: str = "WEM",
    network_region_code: Optional[str] = None,
) -> OpennemDataSet:
    network = network_from_network_code(network_code)
    engine = get_database_engine()
    period: TimePeriod = human_to_period("1Y")
    network = network_from_network_code(network_code)
    interval = human_to_interval("1d")
    units = get_unit("energy_giga")

    query = energy_network_fueltech_daily_query(
        year=year,
        interval=interval,
        network=network,
        network_region=network_region_code,
    )

    # print(query)

    with engine.connect() as c:
        row = list(c.execute(query))

    results_energy = [
        DataQueryResult(
            interval=i[0], group_by=i[1], result=i[2] if len(i) > 1 else None
        )
        for i in row
    ]

    results_market_value = [
        DataQueryResult(
            interval=i[0], group_by=i[1], result=i[3] if len(i) > 1 else None
        )
        for i in row
    ]

    if len(results_energy) < 1:
        raise Exception("No results from query: {}".format(query))

    stats = stats_factory(
        stats=results_energy,
        units=units,
        network=network,
        fueltech_group=True,
        interval=interval,
        region=network.code.lower(),
        period=period,
        code=network.code.lower(),
    )

    stats_market_value = stats_factory(
        stats=results_market_value,
        units=get_unit("market_value"),
        network=network,
        fueltech_group=True,
        interval=interval,
        region=network.code.lower(),
        period=period,
        code=network.code.lower(),
    )

    if stats_market_value:
        stats.data += stats_market_value.data

    return stats


def market_value_year(year: int, network_code: str = "WEM"):
    network = network_from_network_code(network_code)
    engine = get_database_engine()

    units = get_unit("market_value")
    interval = human_to_interval("1d")

    query = market_value_daily_query(network=network, year=year)

    with engine.connect() as c:
        row = list(c.execute(query))

    results = [
        DataQueryResult(
            interval=i[0], result=i[1], group_by=i[2] if len(i) > 1 else None
        )
        for i in row
    ]

    if len(results) < 1:
        raise Exception("No results from query: {}".format(query))

    stats = stats_factory(
        stats=results,
        units=units,
        network=network,
        fueltech_group=True,
        interval=interval,
        region=network.code.lower(),
        code=network.code.lower(),
    )

    return stats


def market_value_all(network_code: str = "WEM"):
    network = network_from_network_code(network_code)
    engine = get_database_engine()

    query = wem_market_value_all_query(network_code="WEM")
    units = get_unit("market_value")
    interval = human_to_interval("1M")

    with engine.connect() as c:
        row = list(c.execute(query))

    results = [
        DataQueryResult(
            interval=i[0], result=i[1], group_by=i[2] if len(i) > 1 else None
        )
        for i in row
    ]

    if len(results) < 1:
        raise Exception("No results from query: {}".format(query))

    stats = stats_factory(
        stats=results,
        units=units,
        network=network,
        fueltech_group=True,
        interval=interval,
        code=network.code,
    )

    return stats




"""

NEMWEB Data ingress into OpenNEM format


"""

import logging
from collections import OrderedDict
from datetime import datetime
from itertools import groupby
from typing import Dict, List, Optional

from sqlalchemy.dialects.postgresql import insert

from opennem.core.networks import NetworkNEM
from opennem.core.normalizers import clean_float, normalize_duid
from opennem.db import SessionLocal, get_database_engine
from opennem.db.models.opennem import BalancingSummary, FacilityScada
from opennem.schema.network import NetworkSchema
from opennem.utils.dates import parse_date
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


def process_case_solutions(table):
    pass


def process_pre_ap_price(table, spider):
    session = SessionLocal()
    engine = get_database_engine()

    if "records" not in table:
        raise Exception("Invalid table no records")

    records = table["records"]

    records_to_store = []

    for record in records:
        trading_interval = parse_date(
            record["SETTLEMENTDATE"], network=NetworkNEM, dayfirst=False
        )

        if not trading_interval:
            continue

        records_to_store.append(
            {
                "network_id": "NEM",
                "created_by": spider.name,
                # "updated_by": None,
                "network_region": record["REGIONID"],
                "trading_interval": trading_interval,
                "price": record["PRE_AP_ENERGY_PRICE"],
            }
        )

    stmt = insert(BalancingSummary).values(records_to_store)
    stmt.bind = engine
    stmt = stmt.on_conflict_do_update(
        index_elements=["trading_interval", "network_id", "network_region"],
        set_={"price": stmt.excluded.price},
    )

    try:
        session.execute(stmt)
        session.commit()
    except Exception as e:
        logger.error("Error inserting records")
        logger.error(e)
        return 0
    finally:
        session.close()

    return len(records_to_store)


def unit_scada_generate_facility_scada(
    records,
    spider=None,
    network: NetworkSchema = NetworkNEM,
    interval_field: str = "SETTLEMENTDATE",
    facility_code_field: str = "DUID",
    power_field: Optional[str] = None,
    energy_field: Optional[str] = None,
    is_forecast: bool = False,
    primary_key_track: bool = False,
    groupby_filter: bool = True,
    limit: int = 0,
) -> List[Dict]:
    created_at = datetime.now()
    primary_keys = []
    return_records = []

    created_by = ""

    if spider and hasattr(spider, "name"):
        created_by = spider.name

    for row in records:

        trading_interval = parse_date(
            row[interval_field], network=network, dayfirst=False
        )

        # if facility_code_field not in row:
        # logger.error("Invalid row no facility_code")
        # continue

        facility_code = normalize_duid(row[facility_code_field])

        if primary_key_track:
            pkey = (trading_interval, facility_code)

            if pkey in primary_keys:
                continue

            primary_keys.append(pkey)

        generated = None

        if power_field and power_field in row:
            generated = clean_float(row[power_field])

        energy = None

        if energy_field and energy_field in row:
            energy = clean_float(row[energy_field])

        __rec = OrderedDict(
            {
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": None,
                "network_id": network.code,
                "trading_interval": trading_interval,
                "facility_code": facility_code,
                "generated": generated,
                "eoi_quantity": energy,
                "is_forecast": is_forecast,
            }
        )

        return_records.append(__rec)

        if limit > 0 and len(return_records) >= limit:
            break

    if not groupby_filter:
        return return_records

    return_records_grouped = {}

    for pk_values, rec_value in groupby(
        return_records,
        key=lambda r: (
            r.get("network_id"),
            r.get("trading_interval"),
            r.get("facility_code"),
        ),
    ):
        if pk_values not in return_records_grouped:
            return_records_grouped[pk_values] = list(rec_value).pop()

    return_records = list(return_records_grouped.values())

    return return_records


def process_unit_scada(table, spider):
    if "records" not in table:
        raise Exception("Invalid table no records")

    records = table["records"]
    item = dict()

    item["table_schema"] = FacilityScada
    item["update_fields"] = ["generated"]
    item["records"] = unit_scada_generate_facility_scada(
        records, spider, power_field="SCADAVALUE", network=NetworkNEM
    )
    item["content"] = None

    return item


def process_unit_solution(table, spider):
    if "records" not in table:
        raise Exception("Invalid table no records")

    records = table["records"]
    item = dict()

    item["table_schema"] = FacilityScada
    item["update_fields"] = ["generated"]
    item["records"] = unit_scada_generate_facility_scada(
        records,
        spider,
        network=NetworkNEM,
        interval_field="SETTLEMENTDATE",
        facility_code_field="DUID",
        power_field="INITIALMW",
    )
    item["content"] = None

    return item


def process_meter_data_gen_duid(table, spider):
    if "records" not in table:
        raise Exception("Invalid table no records")

    records = table["records"]
    item = dict()

    item["table_schema"] = FacilityScada
    item["update_fields"] = ["eoi_generated"]
    item["records"] = unit_scada_generate_facility_scada(
        records,
        spider,
        network=NetworkNEM,
        interval_field="INTERVAL_DATETIME",
        facility_code_field="DUID",
        energy_field="MWH_READING",
    )
    item["content"] = None

    return item


TABLE_PROCESSOR_MAP = {
    "METER_DATA_GEN_DUID": "process_meter_data_gen_duid",
    "DISPATCH_CASE_SOLUTION": "process_case_solutions",
    "DISPATCH_UNIT_SCADA": "process_unit_scada",
    "DISPATCH_UNIT_SOLUTION": "process_unit_solution",
    "DISPATCH_PRE_AP_PRICE": "process_pre_ap_price",
}


class NemwebUnitScadaOpenNEMStorePipeline(object):
    @check_spider_pipeline
    def process_item(self, item, spider=None):

        if "tables" not in item:
            raise Exception("Invalid item - no tables located")

        if not isinstance(item["tables"], dict):
            raise Exception("Invalid item - no tables located")

        tables = item["tables"]

        ret = []

        for table in tables.values():
            if "name" not in table:
                logger.info("Invalid table found")
                continue

            table_name = table["name"]

            if table_name not in TABLE_PROCESSOR_MAP:
                logger.info("No processor for table %s", table_name)
                continue

            process_meth = TABLE_PROCESSOR_MAP[table_name]

            if process_meth not in globals():
                logger.info("Invalid processing function %s", process_meth)
                continue

            logger.info("processing table {}".format(table_name))
            record_item = globals()[process_meth](table, spider=spider)

            ret.append(record_item)

        return ret



from huey import RedisHuey, crontab

from opennem.api.export.tasks import (
    au_export_power,
    wem_export_daily,
    wem_export_monthly,
    wem_export_power,
)
from opennem.exporter.geojson import export_facility_geojson
from opennem.settings import settings

huey = RedisHuey("opennem.exporter", host=settings.cache_url.host)


@huey.periodic_task(crontab(minute="*/5"))
def schedule_wem_export_task():
    wem_export_power()


@huey.periodic_task(crontab(hour="*/1"))
def schedule_wem_export_daily_most_recent():
    wem_export_daily(limit=1)


@huey.periodic_task(crontab(hour="*/4"))
def schedule_wem_export_years():
    wem_export_daily()


@huey.periodic_task(crontab(hour="*/6"))
def schedule_wem_export_all():
    wem_export_monthly()


@huey.periodic_task(crontab(minute="*/5"))
def schedule_export_geojson():
    export_facility_geojson()


@huey.periodic_task(crontab(minute="*/30"))
def schedule_export_au():
    au_export_power()



import logging
from datetime import datetime

from fastapi import HTTPException

from opennem.api.export.controllers import (
    energy_fueltech_daily,
    market_value_all,
    market_value_year,
    power_week,
    weather_daily,
)
from opennem.api.stats.controllers import get_scada_range, stats_factory
from opennem.api.stats.router import (
    energy_network_fueltech_api,
    power_network_fueltech_api,
    price_network_region_api,
)
from opennem.api.stats.schema import DataQueryResult, OpennemDataSet
from opennem.api.time import human_to_interval
from opennem.api.weather.router import station_observations_api
from opennem.core.units import get_unit
from opennem.db import get_database_engine
from opennem.exporter.aws import write_to_s3
from opennem.exporter.local import write_to_local
from opennem.schema.network import NetworkNEM

YEAR_MIN = 2011

logger = logging.getLogger(__name__)


def write_output(path: str, stat_set: OpennemDataSet, is_local: bool = False):
    # @TODO migrate to s3 methods
    byte_count = 0

    if is_local:
        byte_count = write_to_local(path, stat_set.json(exclude_unset=True))
    else:
        byte_count = write_to_s3(path, stat_set.json(exclude_unset=True))

    logger.info("Wrote {} bytes to {}".format(byte_count, path))


def wem_export_power(is_local: bool = False):
    engine = get_database_engine()

    stat_set = power_week(network_code="WEM")

    weather = station_observations_api(
        station_code="009021",
        interval="30m",
        period="7d",
        network_code="WEM",
        engine=engine,
    )

    price = price_network_region_api(
        engine=engine,
        network_code="WEM",
        network_region_code="WEM",
        interval="30m",
        period="7d",
    )

    # demand = wem_demand()

    stat_set.data = stat_set.data + price.data + weather.data

    POWER_ENDPOINT = "/power/wem.json"

    write_output(POWER_ENDPOINT, stat_set, is_local=is_local)


def wem_export_daily(limit: int = None, is_local: bool = False):
    processed_years = 0

    for year in range(datetime.now().year, YEAR_MIN - 1, -1):

        stat_set = None

        stat_set = energy_fueltech_daily(year=year, network_code="WEM")

        try:
            weather = weather_daily(
                station_code="009021", year=year, network_code="WEM",
            )
            stat_set.data += weather.data
        except HTTPException:
            pass

        write_output(
            f"/wem/energy/daily/{year}.json", stat_set, is_local=is_local
        )

        processed_years += 1

        if limit and limit >= processed_years:
            return None


def wem_export_monthly():
    """

        @TODO this is slow atm because of on_energy_sum
    """

    engine = get_database_engine()

    stats = energy_network_fueltech_api(
        network_code="WEM",
        period="all",
        engine=engine,
        interval="1M",
        network_region=None,
    )

    market_value = market_value_all(network_code="WEM")

    stats.data += market_value.data

    write_to_s3("/wem/energy/monthly/all.json", stats.json(exclude_unset=True))


def au_export_power():
    engine = get_database_engine()

    scada_range = get_scada_range(network=NetworkNEM)
    units = get_unit("power")

    __query = """
        SET SESSION TIME ZONE '+10';

        select
            t.trading_interval,
            sum(t.facility_power),
            t.fueltech_code
        from (
            select
                time_bucket_gapfill('30m', trading_interval) AS trading_interval,
                interpolate(
                    max(fs.generated)
                ) as facility_power,
                fs.facility_code,
                ft.code as fueltech_code
            from facility_scada fs
            join facility f on fs.facility_code = f.code
            join fueltech ft on f.fueltech_id = ft.code
            where
                fs.trading_interval <= {date_end}
                and fs.trading_interval >= {date_end}::timestamp - '7 days'::interval
                and fs.network_id in ('NEM', 'WEM')
                and f.fueltech_id is not null
            group by 1, 3, 4
        ) as t
        group by 1, 3
        order by 1 desc
    """

    query = __query.format(date_end=scada_range.get_end_sql(),)

    with engine.connect() as c:
        results = list(c.execute(query))

    stats = [
        DataQueryResult(
            interval=i[0], result=i[1], group_by=i[2] if len(i) > 1 else None
        )
        for i in results
    ]

    interval = human_to_interval("30m")

    stats = stats_factory(
        stats,
        code="au",
        network=NetworkNEM,
        interval=interval,
        units=units,
        fueltech_group=True,
    )

    write_to_s3("/power/au.json", stats.json(exclude_unset=True))


if __name__ == "__main__":
    # au_export_power()
    wem_export_power(is_local=True)
    wem_export_daily(limit=1, is_local=True)
    # wem_export_monthly()
